function ObjectionStartup() {}

ObjectionStartup.prototype = {
	classID: Components.ID("{10060352-006A-11DD-84D9-524756D89593}"),
	contractID: "@objection/startup;1",
	classDescription: "Objection Startup",

	QueryInterface: function(aIID)
	{
		if(!aIID.equals(CI.nsISupports) && !aIID.equals(CI.nsIObserver))
		throw CR.NS_ERROR_NO_INTERFACE;
		return this;
	},

	observe: function(aSubject, aTopic, aData)
	{
		switch(aTopic)
		{
			case "xpcom-startup":
				// this is run very early, right after XPCOM is initialized, but before
				// user profile information is applied. Register ourselves as an observer
				// for 'profile-after-change' and 'quit-application'.
				var obsSvc = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
				obsSvc.addObserver(this, "profile-after-change", false);
				obsSvc.addObserver(this, "quit-application", false);
				break;

			case "profile-after-change":
				this.removeAutomatic('open');
				break;

			case "quit-application":
				this.removeAutomatic('close');
				break;

			default:
				throw Components.Exception("Unknown topic: " + aTopic);
		}
	},

	removeAutomatic: function(timeEvent)
	{
		// get a reference to the prompt service component. For debugging.
		var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
		
		var nsILocalFile = new Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile");
		var preferences = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection.");
		var automaticRemove = 'never';
		
		try
		{
			var automaticRemove = preferences.getCharPref("automaticRemove");
		}
		catch (err)	{}
		if (timeEvent == automaticRemove)
		{
			// Init the File/Directory objects
			var flashAppDir = new nsILocalFile();
			var flashSettingDir = new nsILocalFile();
			var flashSettingFile = new nsILocalFile();
			var flashObjectDir = new nsILocalFile();
		
			// Get the Flash data directory and delete files
			if (preferences.prefHasUserValue("flashAppDir") && typeof(preferences.getCharPref("flashAppDir")) != 'undefined')
			{
				var flashAppDirPref = preferences.getCharPref("flashAppDir");
				flashAppDir.initWithPath(flashAppDirPref);
				if (flashAppDir.exists() && flashAppDir.isDirectory())
				{
					flashSettingDir.initWithFile(flashAppDir);
					flashSettingDir.appendRelativePath('macromedia.com');
					flashSettingDir.appendRelativePath('support');
					flashSettingDir.appendRelativePath('flashplayer');
					flashSettingDir.appendRelativePath('sys');
					if (flashSettingDir.exists() && flashSettingDir.isDirectory())
					{
						// Delete all lso directories (including the files)
						try
						{
							var entries = flashSettingDir.directoryEntries;
							while(entries.hasMoreElements())
							{
								var entry = entries.getNext();
								entry.QueryInterface(Components.interfaces.nsIFile);
								if (entry.isDirectory())
								{
									try
									{
										entry.remove(true);
									} catch (er) {}
								}
							}
						} catch(er)	{}


					}
					var automaticRemove = false;
					try
					{
						automaticRemoveSite = preferences.getBoolPref("automaticRemoveSite");
					}
					catch (err) {}
					// Remove the site settings as well, if requested
					if (automaticRemoveSite)
					{
						// Locate settings directories
						var flashObjectDirSet = false;
						var flashObjectDir = new nsILocalFile();
						flashObjectDir.initWithFile(flashAppDir);
						flashObjectDir.appendRelativePath('#SharedObjects');
						if (flashObjectDir.exists() && flashObjectDir.isDirectory())
						{
							try
							{
								var entries = flashObjectDir.directoryEntries;
								while(entries.hasMoreElements())
								{
									var entry = entries.getNext();
									entry.QueryInterface(Components.interfaces.nsIFile);
									if (entry.isDirectory())
									{
										flashObjectDir.initWithFile(entry);
										flashObjectDirSet = true;
										break;
									}
								}
							} catch(er) {}
							// Delete settings directories
							if (flashObjectDirSet)
							{
								try
								{
									var entries = flashObjectDir.directoryEntries;
									while(entries.hasMoreElements())
									{
										var entry = entries.getNext();
										entry.QueryInterface(Components.interfaces.nsIFile);
										if (entry.isDirectory())
										{
											try
											{
												entry.remove(true);
											} catch (er) {}
										}
									}
								} catch(er)	{}
							}
						}
					}
//					promptService.alert(null,'Objection','so far...');
				}
			}
		}
	},
};


// constructors for objects we want to XPCOMify
var objects = [ObjectionStartup];

/*
 * Registration code.
 *
 */

const CI = Components.interfaces, CC = Components.classes, CR = Components.results;

const MY_OBSERVER_NAME = "ObjectionStartup";

function FactoryHolder(aObj) {
  this.CID        = aObj.prototype.classID;
  this.contractID = aObj.prototype.contractID;
  this.className  = aObj.prototype.classDescription;
  this.factory = {
    createInstance: function(aOuter, aIID) {
      if(aOuter)
        throw CR.NS_ERROR_NO_AGGREGATION;
      return (new this.constructor).QueryInterface(aIID);
    }
  };
  this.factory.constructor = aObj;
}

var gModule = {
  registerSelf: function (aComponentManager, aFileSpec, aLocation, aType)
  {
    aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
    for (var key in this._objects) {
      var obj = this._objects[key];
      aComponentManager.registerFactoryLocation(obj.CID, obj.className,
        obj.contractID, aFileSpec, aLocation, aType);
    }

    var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
    catman.addCategoryEntry("xpcom-startup", MY_OBSERVER_NAME,
      ObjectionStartup.prototype.contractID, true, true);
    catman.addCategoryEntry("xpcom-shutdown", MY_OBSERVER_NAME,
      ObjectionStartup.prototype.contractID, true, true);
  },

  unregisterSelf: function(aCompMgr, aFileSpec, aLocation) {
    var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
    catman.deleteCategoryEntry("xpcom-startup", MY_OBSERVER_NAME, true);

    aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
    for (var key in this._objects) {
      var obj = this._objects[key];
      aComponentManager.unregisterFactoryLocation(obj.CID, aFileSpec);
    }
  },

  getClassObject: function(aComponentManager, aCID, aIID) {
    if (!aIID.equals(CI.nsIFactory)) throw CR.NS_ERROR_NOT_IMPLEMENTED;

    for (var key in this._objects) {
      if (aCID.equals(this._objects[key].CID))
        return this._objects[key].factory;
    }
   
    throw CR.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aComponentManager) {
    return true;
  },

  _objects: {} //FactoryHolder
};

function NSGetModule(compMgr, fileSpec)
{
  for(var i in objects)
    gModule._objects[i] = new FactoryHolder(objects[i]);
  return gModule;
} 
